Vehicle: HotStuff4
Designer: Magus
Body: Hexo
Engine: Fuel Cell
W#0: T1 Heavy Machine Gun
W#1: T1 Heavy Machine Gun
W#2: T1 Heavy Machine Gun
Armor Type: Titanium
Front: 18
Back: 12
Left: 16
Right: 16
Top: 12
Bottom: 6
Special: Console
Special: Mapper
Special: HU Display
Special: Stealth
Special: Tac Link
Special: Camo
Special: Rdf
Heat sinks: 2
Suspension: Active
Treads: Normal
Bumpers: Retro
