Vehicle: Katana3
Designer: Katz
Body: Malice
Engine: Fuel Cell
W#0: F ML
W#1: T1 HPR
W#2: T1 HPR
W#3: T1 HPR
W#4: T1 HPR
Armor Type: Titanium
Front: 71
Back: 65
Left: 62
Right: 62
Top: 25
Bottom: 21
Special: Console
Special: Mapper
Special: Radar
Heat sinks: 8
Suspension: Active
Treads: Normal
Bumpers: Retro
