/* game.h - part of xtank */

#ifndef _GAME_H_
#define _GAME_H_


/* the different games that can be played */
typedef enum {
    COMBAT_GAME, WAR_GAME, ULTIMATE_GAME, CAPTURE_GAME, RACE_GAME, STQ_GAME
} Game;


#endif ndef _GAME_H_
