Vehicle: Gnat2
Designer: Katz
Body: Malice
Engine: Fuel Cell
W#0: T1 Acid Sprayer
W#1: T1 Acid Sprayer
W#2: T1 Acid Sprayer
W#3: T1 Acid Sprayer
W#4: T1 Heavy Rkt Launcher
W#5: T1 Heavy Rkt Launcher
Armor Type: Titanium
Front: 80
Back: 76
Left: 72
Right: 72
Top: 31
Bottom: 29
Special: Console
Special: Mapper
Special: Radar
Special: Ram Plate
Heat sinks: 4
Suspension: Active
Treads: Normal
Bumpers: Retro
