Vehicle: GnatRam2
Designer: Katz
Body: Tiger
Engine: Fusion
W#0: T1 Acid Sprayer
W#1: T1 Acid Sprayer
W#2: T1 Acid Sprayer
W#3: T1 Acid Sprayer
W#4: T1 Acid Sprayer
W#5: T1 Heavy Autocannon
Armor Type: Composite
Front: 225
Back: 100
Left: 121
Right: 121
Top: 71
Bottom: 51
Special: Console
Special: Mapper
Special: Radar
Special: Repair
Special: Ram Plate
Heat sinks: 2
Suspension: Active
Treads: Normal
Bumpers: Retro
