Vehicle: Falcon
Designer: mark
Body: Tiger
Engine: Fusion
W#0: R Anti-Radiation
W#1: T1 Pocket Rocket
W#2: T1 Pocket Rocket
Armor Type: Titanium
Front: 110
Back: 65
Left: 65
Right: 65
Top: 50
Bottom: 0
Special: Console
Special: Mapper
Special: Radar
Special: Repair
Special: Ram Plate
Special: Deep Radar
Special: Stealth
Special: Navigation
Special: New Radar
Special: Tac Link
Heat sinks: 7
Suspension: Active
Treads: Hover
Bumpers: Retro

