Vehicle: GnatRam
Designer: Katz
Body: Malice
Engine: Fuel Cell
W#0: T1 Acid Sprayer
W#1: T1 Acid Sprayer
W#2: T1 Acid Sprayer
W#3: T1 Acid Sprayer
W#4: T1 Acid Sprayer
W#5: T1 Acid Sprayer
Armor Type: Composite
Front: 201
Back: 100
Left: 126
Right: 126
Top: 36
Bottom: 51
Special: Console
Special: Mapper
Special: Radar
Special: Ram Plate
Heat sinks: 0
Suspension: Active
Treads: Normal
Bumpers: Retro
