/*
** Xtank
**
** Copyright 1988 by Terry Donahue
**
** disc.h
*/

/*
$Author: rpotter $
$Id: disc.h,v 2.3 1991/02/10 13:50:21 rpotter Exp $

$Log: disc.h,v $
 * Revision 2.3  1991/02/10  13:50:21  rpotter
 * bug fixes, display tweaks, non-restart fixes, header reorg.
 *
 * Revision 2.2  91/01/20  09:57:34  rpotter
 * complete rewrite of vehicle death, other tweaks
 * 
 * Revision 2.1  91/01/17  07:11:12  rpotter
 * lint warnings and a fix to update_vector()
 * 
 * Revision 2.0  91/01/17  02:09:17  rpotter
 * small changes
 * 
 * Revision 1.1  90/12/29  21:02:18  aahz
 * Initial revision
 * 
*/

#define DISC_ORBIT_SQ	1600.0
#define DISC_SLOW_SPEED 5.0
#define DISC_MED_SPEED  15.0
#define DISC_FAST_SPEED 25.0
