Vehicle: Disgorge5
Designer: Senft
Body: Marauder
Engine: Fuel Cell
W#0: T1 Acid Sprayer
W#1: T1 Acid Sprayer
W#2: T2 Acid Sprayer
W#3: T2 Acid Sprayer
Armor Type: Composite
Front: 187
Back: 145
Left: 210
Right: 210
Top: 55
Bottom: 33
Special: Console
Special: Mapper
Special: New Radar
Special: Tac Link
Heat sinks: 0
Suspension: Active
Treads: Smooth
Bumpers: Retro
