Vehicle: HotStuff1
Designer: Stripes
Body: Psycho
Engine: Fuel Cell
W#0: T1 Unguided Missle
W#1: T1 Unguided Missle
Armor Type: Titanium
Front: 40
Back: 40
Left: 40
Right: 40
Top: 0
Bottom: 0
Special: Console
Special: Mapper
Special: Repair
Special: HU Display
Special: Stealth
Special: Navigation
Special: New Radar
Special: Tac Link
Special: Camo
Special: Rdf
Heat sinks: 2
Suspension: Active
Treads: Hover
Bumpers: Retro
