Vehicle: Wimpy2
Designer: Stripes
Body: Lightcycle
Engine: Super Electric
W#0: F Machine Gun
Armor Type: Kevlar
Front: 10
Back: 4
Left: 18
Right: 18
Top: 0
Bottom: 0
Special: Console
Special: Mapper
Special: HU Display
Special: Tac Link
Special: Camo
Special: Rdf
Heat sinks: 0
Suspension: Active
Treads: Chained
Bumpers: Retro
