Vehicle: BASE3
Designer: Stripes
Body: Panzy
Engine: Small Electric
W#0: T1 Heavy Machine Gun
W#1: T1 Heavy Machine Gun
W#2: T2 Light Machine Gun
W#3: T3 Flame Thrower
W#4: T4 Machine Gun
W#5: T4 Machine Gun
Armor Type: New Tungsten
Front: 88
Back: 88
Left: 88
Right: 87
Top: 0
Bottom: 0
Special: Console
Special: Mapper
Special: Repair
Special: HU Display
Special: Camo
Special: Rdf
Heat sinks: 10
Suspension: Active
Treads: Smooth
Bumpers: Retro
