Vehicle: DaveDude4
Designer: ratner
Body: Tornado
Engine: Fuel Cell
W#0: T1 Acid Sprayer
W#1: T1 Acid Sprayer
W#2: T1 Acid Sprayer
W#3: F Acid Sprayer     
W#4: F Acid Sprayer    
W#5: F Acid Sprayer
Armor Type: Kevlar
Front: 385
Back: 75
Left: 100
Right: 100
Top: 56
Bottom: 0
Special: Console
Special: Mapper
Special: Ram Plate
Special: Navigation
Special: New Radar
Special: Tac Link
Heat sinks: 0
Suspension: Active
Treads: Hover
Bumpers: Retro
