Vehicle: Lewis
Designer: lbruck
Body: Marauder
Engine: Fuel Cell
W#0: T1 Heavy Autocannon
W#1: T2 Heavy Autocannon
Armor Type: Titanium
Front: 75
Back: 75
Left: 50
Right: 50
Top: 30
Bottom: 20
Special: Console
Special: Mapper
Special: Radar
Special: HU Display
Special: Stealth
Special: New Radar
Special: Tac Link
Special: Camo
Special: Rdf
Heat sinks: 10
Suspension: Normal
Treads: Normal
Bumpers: Normal
