Vehicle: HotStuff2
Designer: Stripes
Body: Psycho
Engine: Fuel Cell
W#0: T1 Heavy Machine Gun
W#1: T1 Heavy Machine Gun
W#2: T1 Heavy Machine Gun
W#3: T1 Heavy Machine Gun
W#4: T1 Heavy Machine Gun
W#5: T1 Heavy Machine Gun
Armor Type: Titanium
Front: 43
Back: 42
Left: 42
Right: 42
Top: 0
Bottom: 0
Special: Console
Special: Mapper
Special: HU Display
Special: Stealth
Special: Navigation
Special: Tac Link
Special: Camo
Special: Rdf
Heat sinks: 4
Suspension: Active
Treads: Hover
Bumpers: Rubber
