Vehicle: Disgorge2
Designer: Senft
Body: Marauder
Engine: Fuel Cell
W#0: T1 Acid Sprayer
W#1: T1 Acid Sprayer
W#2: T2 Acid Sprayer
W#3: T2 Acid Sprayer
W#4: R Mine Layer
W#5: L Oil Slick
Armor Type: Composite
Front: 121
Back: 180
Left: 212
Right: 212
Top: 50
Bottom: 0
Special: Console
Special: Mapper
Special: New Radar
Special: Tac Link
Heat sinks: 0
Suspension: Active
Treads: Hover
Bumpers: Retro
