Vehicle: foo2
Designer: Smaug
Body: Delta
Engine: Fuel Cell
W#0: T1 Pulse Rifle
W#1: T2 Pulse Rifle
W#2: T1 Heavy Machine Gun
W#3: T2 Heavy Machine Gun
W#4: B Mine Layer
Armor Type: Kevlar
Front: 100
Back: 50
Left: 50
Right: 50
Top: 50
Bottom: 50
Special: Console
Special: Mapper
Special: Radar
Special: Ram Plate
Special: HU Display
Special: Stealth
Special: New Radar
Special: Tac Link
Special: Camo
Special: Rdf
Heat sinks: 2
Suspension: Active
Treads: Normal
Bumpers: Normal
