Vehicle: Trike3
Designer: skeeve
Body: Trike
Engine: Super Electric
W#0: F Acid Sprayer
Armor Type: Titanium
Front: 25
Back: 10
Left: 12
Right: 12
Top: 0
Bottom: 0
Special: Console
Special: Mapper
Special: Radar
Special: Ram Plate
Heat sinks: 0
Suspension: Active
Treads: Hover
Bumpers: None
