Vehicle: Disgorge4
Designer: Senft
Body: Marauder
Engine: Fusion
W#0: T1 Acid Sprayer
W#1: T1 Acid Sprayer
W#2: T1 Acid Sprayer
W#3: T2 Heavy Machine Gun
W#4: T2 Heavy Machine Gun
W#5: T2 Heavy Machine Gun
Armor Type: Composite
Front: 114
Back: 100
Left: 100
Right: 100
Top: 114
Bottom: 30
Special: Console
Special: Mapper
Special: Ram Plate
Special: New Radar
Special: Tac Link
Heat sinks: 3
Suspension: Active
Treads: Smooth
Bumpers: Retro
