Vehicle: HotStuff3
Designer: Magus
Body: Hexo
Engine: Fuel Cell
W#0: T1 Heavy Machine Gun
W#1: T1 Heavy Machine Gun
W#2: T1 Light Machine Gun
Armor Type: Composite
Front: 20
Back: 20
Left: 18
Right: 18
Top: 14
Bottom: 10
Special: Console
Special: Mapper
Special: HU Display
Special: Stealth
Special: Tac Link
Special: Camo
Special: Rdf
Heat sinks: 2
Suspension: Active
Treads: Smooth
Bumpers: Retro
