Vehicle: FC25
Designer: senft
Body: Rhino
Engine: Fuel Cell
W#0: T1 Pocket Rocket
W#1: T1 Pocket Rocket
W#2: T1 Pocket Rocket
W#3: T2 Pocket Rocket
W#4: T2 Pocket Rocket
W#5: T2 Pocket Rocket
Armor Type: Kevlar
Front: 65
Back: 65
Left: 65
Right: 65
Top: 40
Bottom: 0
Special: Console
Special: Mapper
Special: HU Display
Special: New Radar
Special: Tac Link
Heat sinks: 13
Suspension: Active
Treads: Hover
Bumpers: Retro
