/* Un-comment to use a wimpy debug version of malloc */
/*
$Author: rpotter $
$Id: malloc.h,v 2.3 1991/02/10 13:51:09 rpotter Exp $

$Log: malloc.h,v $
 * Revision 2.3  1991/02/10  13:51:09  rpotter
 * bug fixes, display tweaks, non-restart fixes, header reorg.
 *
 * Revision 2.2  91/01/20  09:58:25  rpotter
 * complete rewrite of vehicle death, other tweaks
 * 
 * Revision 2.1  91/01/17  07:12:16  rpotter
 * lint warnings and a fix to update_vector()
 * 
 * Revision 2.0  91/01/17  02:10:00  rpotter
 * small changes
 * 
 * Revision 1.1  90/12/29  21:02:44  aahz
 * Initial revision
 * 
*/

/* #define malloc(size)	(my_malloc(size)) */
