Vehicle: Gnat5
Designer: Katz
Body: Malice
Engine: Fuel Cell
W#0: T1 Acid Sprayer
W#1: T1 Acid Sprayer
W#2: T1 Acid Sprayer
W#3: T1 Acid Sprayer
W#4: T1 Heat Seeker
W#5: T1 Heat Seeker
Armor Type: Composite
Front: 103
Back: 99
Left: 96
Right: 96
Top: 36
Bottom: 36
Special: Console
Special: Mapper
Special: Radar
Special: Ram Plate
Heat sinks: 2
Suspension: Active
Treads: Normal
Bumpers: Retro
