Vehicle: Gnat3
Designer: Katz
Body: Malice
Engine: Fuel Cell
W#0: T1 Acid Sprayer
W#1: T1 Acid Sprayer
W#2: T1 Acid Sprayer
W#3: T1 Heavy Machine Gun
W#4: T1 Heavy Machine Gun
W#5: T1 Heavy Machine Gun
Armor Type: Composite
Front: 136
Back: 132
Left: 128
Right: 128
Top: 36
Bottom: 41
Special: Console
Special: Mapper
Special: Radar
Special: Ram Plate
Heat sinks: 3
Suspension: Active
Treads: Normal
Bumpers: Retro
