Vehicle: Disgorge8
Designer: Senft
Body: Marauder
Engine: Fusion
W#0: T1 Acid Sprayer
W#1: T1 Acid Sprayer
W#2: T2 Acid Sprayer
W#3: T2 Acid Sprayer
Armor Type: Kevlar
Front: 186
Back: 180
Left: 231
Right: 231
Top: 48
Bottom: 24
Special: Console
Special: Mapper
Special: New Radar
Special: Tac Link
Heat sinks: 0
Suspension: Active
Treads: Smooth
Bumpers: Retro
