Vehicle: Gnat4
Designer: Katz
Body: Malice
Engine: Fuel Cell
W#0: T1 Acid Sprayer
W#1: T1 Acid Sprayer
W#2: T1 Acid Sprayer
W#3: T1 Acid Sprayer
W#4: T1 Acid Sprayer
W#5: B Mine Layer
Armor Type: Composite
Front: 137
Back: 130
Left: 131
Right: 131
Top: 36
Bottom: 46
Special: Console
Special: Mapper
Special: Radar
Special: Ram Plate
Heat sinks: 0
Suspension: Active
Treads: Normal
Bumpers: Retro
