Vehicle: Katana4
Designer: Katz
Body: Malice
Engine: Fuel Cell
W#0: F ML
W#1: T1 AS
W#2: T1 AS
W#3: T1 AS
W#4: T1 AS
W#5: T1 AS
Armor Type: Composite
Front: 146
Back: 141
Left: 137
Right: 137
Top: 43
Bottom: 21
Special: Console
Special: Mapper
Special: Radar
Heat sinks: 0
Suspension: Active
Treads: Normal
Bumpers: Retro
