Vehicle: Katana1
Designer: Katz
Body: Malice
Engine: Fuel Cell
W#0: F ML
W#1: T1 HRL
W#2: T1 HRL
Armor Type: Titanium
Front: 67
Back: 62
Left: 59
Right: 59
Top: 25
Bottom: 21
Special: Console
Special: Mapper
Special: Radar
Heat sinks: 6
Suspension: Active
Treads: Normal
Bumpers: Retro
