Vehicle: Disgorge7
Designer: Senft
Body: Marauder
Engine: Fuel Cell
W#0: T1 Heavy Machine Gun
W#1: T1 Heavy Machine Gun
W#2: T1 Heavy Machine Gun
W#3: T2 Heavy Machine Gun
W#4: T2 Heavy Machine Gun
W#5: T2 Heavy Machine Gun
Armor Type: Titanium
Front: 108
Back: 75
Left: 131
Right: 131
Top: 80
Bottom: 0
Special: Console
Special: Mapper
Special: Ram Plate
Special: New Radar
Special: Tac Link
Heat sinks: 8
Suspension: Active
Treads: Hover
Bumpers: Retro
