Vehicle: TagRam4
Designer: senft
Body: Hexo
Engine: Fuel Cell
W#0: F Acid Sprayer
W#1: F Acid Sprayer
W#2: F Acid Sprayer
W#3: F Acid Sprayer
W#4: F Acid Sprayer
Armor Type: Kevlar
Front: 50
Back: 10
Left: 16
Right: 16
Top: 4
Bottom: 6
Special: Console
Special: Mapper
Special: Ram Plate
Special: New Radar
Special: Tac Link
Heat sinks: 0
Suspension: Active
Treads: Spiked
Bumpers: Normal
