Vehicle: foo
Designer: Smaug
Body: Marauder
Engine: Fuel Cell
W#0: T1 Anti-Radiation
W#1: T2 Anti-Radiation
Armor Type: New Tungsten
Front: 60
Back: 50
Left: 40
Right: 40
Top: 20
Bottom: 18
Special: Console
Special: Mapper
Special: Radar
Special: HU Display
Special: Stealth
Special: New Radar
Special: Tac Link
Special: Camo
Special: Rdf
Heat sinks: 0
Suspension: Active
Treads: Smooth
Bumpers: Normal
