Vehicle: DD5
Designer: ratner
Body: Tornado
Engine: Fuel Cell
W#0: T1 Acid Sprayer
W#1: T1 Acid Sprayer
W#2: T1 Acid Sprayer
W#3: F Light Machine Gun
Armor Type: Kevlar
Front: 385
Back: 75
Left: 125
Right: 125
Top: 56
Bottom: 56
Special: Console
Special: Mapper
Special: Ram Plate
Special: New Radar
Special: Tac Link
Heat sinks: 0
Suspension: Active
Treads: Smooth
Bumpers: Retro
