Vehicle: Minute Man
Designer: Gordon
Body: Medusa
Engine: Small Electric
W#0: T1 Tactical Nuke
W#1: T1 Tactical Nuke
W#2: T2 Heavy Machine Gun
W#3: T2 Heavy Machine Gun
W#4: T3 Tactical Nuke
W#5: T3 Tactical Nuke
Armor Type: Titanium
Front: 80
Back: 80
Left: 80
Right: 80
Top: 45
Bottom: 45
Special: Console
Special: Mapper
Special: Repair
Special: New Radar
Special: Tac Link
Special: Camo
Heat sinks: 9
Suspension: Light
Treads: Spiked
Bumpers: None
