Vehicle: FaceOff
Designer: NedHead
Body: Spider
Engine: Fuel Cell
W#0: T1 Heat Seeker
W#1: T1 Autocannon
Armor Type: Titanium
Front: 11
Back: 11
Left: 10
Right: 10
Top: 8
Bottom: 0
Special: Console
Special: Mapper
Special: Repair
Special: Ram Plate
Special: Deep Radar
Special: Stealth
Special: Navigation
Special: New Radar
Special: Tac Link
Heat sinks: 0
Suspension: Active
Treads: Smooth
Bumpers: Retro
