Vehicle: GnatRacer
Designer: Katz
Body: Malice
Engine: Fusion
W#0: B Mine Layer
W#1: T1 Heavy Machine Gun
W#2: T1 Heavy Machine Gun
Armor Type: Composite
Front: 108
Back: 104
Left: 102
Right: 102
Top: 22
Bottom: 41
Special: Console
Special: Mapper
Special: Radar
Special: Ram Plate
Heat sinks: 2
Suspension: Active
Treads: Chained
Bumpers: Retro
