/*
** Xtank
**
** Copyright 1988 by Terry Donahue
**
** obj3d.h
*/

#define MAX_BUF 256

/* Types of face visibility */
#define ALWAYS     0
#define NEVER      1
#define SOMETIMES  2

typedef unsigned char Byte;

#include "/mit/games/src/vax/xtank/Src/3d.h"

extern char *malloc();
