Vehicle: Nukem
Designer: Wildman
Body: Malice
Engine: Fusion
W#0: T1 Tactical Nuke
Armor Type: Composite
Front: 85
Back: 85
Left: 85
Right: 85
Top: 85
Bottom: 85
Special: Console
Special: Mapper
Special: Radar
Special: Ram Plate
Heat sinks: 1
Suspension: Active
Treads: Normal
Bumpers: Retro
