/***************************\
* globals.h - part of XTank *
\***************************/

/* these are in main.c */
extern int num_veh;
extern Vehicle actual_vehicles[];
extern int num_veh_alive;
extern Vehicle *live_vehicles[];
extern int num_veh_dead;
extern Vehicle *dead_vehicles[];
