Vehicle: Disgorge9
Designer: senft
Body: Medusa
Engine: Fuel Cell
W#0: T1 Pocket Rocket
W#1: T2 Pocket Rocket
W#2: T3 Acid Sprayer
W#3: T3 Acid Sprayer
W#4: T3 Acid Sprayer
W#5: L OS
Armor Type: Composite
Front: 190
Back: 68
Left: 155
Right: 155
Top: 64
Bottom: 0
Special: Console
Special: Mapper
Special: New Radar
Special: Tac Link
Heat sinks: 6
Suspension: Active
Treads: Hover
Bumpers: Retro
