Vehicle: Disgorge
Designer: Senft
Body: Marauder
Engine: Large Combustion
W#0: T1 Acid Sprayer
W#1: T1 Acid Sprayer
W#2: T2 Acid Sprayer
W#3: T2 Acid Sprayer
W#4: L Mine Layer
W#5: R Mine Layer
Armor Type: Composite
Front: 154
Back: 126
Left: 200
Right: 200
Top: 55
Bottom: 0
Special: Console
Special: Mapper
Special: New Radar
Special: Tac Link
Heat sinks: 1
Suspension: Heavy
Treads: Hover
Bumpers: Retro
