Vehicle: Katana2
Designer: Katz
Body: Malice
Engine: Fuel Cell
W#0: F ML
W#1: T1 HMG
W#2: T1 HMG
W#3: T1 HMG
W#4: T1 HMG
Armor Type: Titanium
Front: 84
Back: 79
Left: 76
Right: 76
Top: 37
Bottom: 21
Special: Console
Special: Mapper
Special: Radar
Heat sinks: 7
Suspension: Active
Treads: Normal
Bumpers: Retro
