Vehicle: foo1
Designer: Smaug
Body: Disk
Engine: Fuel Cell
W#0: T1 Flame Thrower
W#1: T2 Flame Thrower
W#2: T1 Flame Thrower
W#3: T2 Flame Thrower
Armor Type: Titanium
Front: 100
Back: 75
Left: 75
Right: 75
Top: 50
Bottom: 25
Special: Console
Special: Mapper
Special: Radar
Special: HU Display
Special: Stealth
Special: New Radar
Special: Tac Link
Special: Camo
Special: Rdf
Heat sinks: 4
Suspension: Active
Treads: Smooth
Bumpers: Rubber
