Vehicle: BurntOut
Designer: Stripes
Body: Marauder
Engine: Fusion
W#0: T1 Flame Thrower
W#1: T1 Flame Thrower
W#2: T1 Flame Thrower
W#3: T2 Flame Thrower
W#4: T2 Flame Thrower
W#5: T2 Flame Thrower
Armor Type: Composite
Front: 100
Back: 80
Left: 80
Right: 80
Top: 30
Bottom: 0
Special: Console
Special: Mapper
Special: Ram Plate
Special: New Radar
Special: Tac Link
Heat sinks: 6
Suspension: Active
Treads: Normal
Bumpers: Retro
