Vehicle: Asticate
Designer: senft
Body: Malice
Engine: Fuel Cell
W#0: T1 Heavy Machine Gun
W#1: T1 Heavy Machine Gun
W#2: T1 Heavy Machine Gun
Armor Type: Titanium
Front: 130
Back: 80
Left: 100
Right: 100
Top: 24
Bottom: 32
Special: Console
Special: Mapper
Special: New Radar
Special: Tac Link
Heat sinks: 6
Suspension: Active
Treads: Smooth
Bumpers: Normal
